EXSWHTEC-350 - Implement tests for hipModuleLaunchCooperativeKernel #429

Change-Id: I42342c7d4cceed67990f603cd83473fe76e7f856


[ROCm/hip-tests commit: 303836a7fd]
Этот коммит содержится в:
Mirza Halilčević
2023-12-28 14:47:48 +01:00
коммит произвёл Rakesh Roy
родитель 29c22f1558
Коммит bb08c70c4e
5 изменённых файлов: 453 добавлений и 0 удалений
+7
Просмотреть файл
@@ -179,6 +179,13 @@ THE SOFTWARE.
* @}
*/
/**
* @defgroup ModuleTest Module Management
* @{
* This section describes the module management types & functions of HIP runtime API.
* @}
*/
/**
* @defgroup TextureTest Texture Management
* @{
+2
Просмотреть файл
@@ -29,6 +29,8 @@ set(TEST_SRC
hipModuleLaunchKernel.cc
hipModuleGetGlobal.cc
hipModuleGetTexRef.cc
hipModuleLaunchCooperativeKernel.cc
hipModuleLaunchCooperativeKernelMultiDevice.cc
)
add_custom_command(OUTPUT ${CMAKE_CURRENT_BINARY_DIR}/get_function_module.code
+211
Просмотреть файл
@@ -0,0 +1,211 @@
/*
Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
/**
* @addtogroup hipModuleLaunchCooperativeKernel hipModuleLaunchCooperativeKernel
* @{
* @ingroup ModuleTest
* `hipModuleLaunchCooperativeKernel(hipFunction_t f, unsigned int gridDimX, unsigned int gridDimY,
* unsigned int gridDimZ, unsigned int blockDimX, unsigned int blockDimY, unsigned int blockDimZ,
* unsigned int sharedMemBytes, hipStream_t stream, void ** kernelParams)` -
* Launches kernel f with launch parameters and shared memory on stream with arguments passed to
* kernelParams, where thread blocks can cooperate and synchronize as they execute.
*/
#include <hip_test_common.hh>
#include <resource_guards.hh>
#include <utils.hh>
#include "hip_module_launch_kernel_common.hh"
/**
* Test Description
* ------------------------
* - Tests `hipModuleLaunchCooperativeKernel` for a cooperative kernel with no parameters, and for
* a normal kernel with parameters.
* Test source
* ------------------------
* - unit/module/hipModuleLaunchCooperativeKernel.cc
* Test requirements
* ------------------------
* - HIP_VERSION >= 5.5
*/
TEST_CASE("Unit_hipModuleLaunchCooperativeKernel_Positive_Basic") {
if (!DeviceAttributesSupport(0, hipDeviceAttributeCooperativeLaunch)) {
HipTest::HIP_SKIP_TEST("CooperativeLaunch not supported");
return;
}
SECTION("Cooperative kernel with no arguments") {
hipFunction_t f = GetKernel(mg.module(), "CoopKernel");
HIP_CHECK(hipModuleLaunchCooperativeKernel(f, 2, 2, 1, 1, 1, 1, 0, nullptr, nullptr));
HIP_CHECK(hipDeviceSynchronize());
}
SECTION("Kernel with arguments using kernelParams") {
hipFunction_t f = GetKernel(mg.module(), "Kernel42");
LinearAllocGuard<int> result_dev(LinearAllocs::hipMalloc, sizeof(int));
HIP_CHECK(hipMemset(result_dev.ptr(), 0, sizeof(*result_dev.ptr())));
int* result_ptr = result_dev.ptr();
void* kernel_args[1] = {&result_ptr};
HIP_CHECK(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, 1, 1, 0, nullptr, kernel_args));
int result = 0;
HIP_CHECK(hipMemcpy(&result, result_dev.ptr(), sizeof(result), hipMemcpyDefault));
REQUIRE(result == 42);
}
}
/**
* Test Description
* ------------------------
* - Positive parameters test for `hipModuleLaunchCooperativeKernel`.
* Test source
* ------------------------
* - unit/module/hipModuleLaunchCooperativeKernel.cc
* Test requirements
* ------------------------
* - HIP_VERSION >= 5.5
*/
TEST_CASE("Unit_hipModuleLaunchCooperativeKernel_Positive_Parameters") {
if (!DeviceAttributesSupport(0, hipDeviceAttributeCooperativeLaunch)) {
HipTest::HIP_SKIP_TEST("CooperativeLaunch not supported");
return;
}
hipFunction_t f = GetKernel(mg.module(), "NOPKernel");
SECTION("blockDim.x == maxBlockDimX") {
const unsigned int x = GetDeviceAttribute(0, hipDeviceAttributeMaxBlockDimX);
HIP_CHECK(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, x, 1, 1, 0, nullptr, nullptr));
}
SECTION("blockDim.y == maxBlockDimY") {
const unsigned int y = GetDeviceAttribute(0, hipDeviceAttributeMaxBlockDimY);
HIP_CHECK(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, y, 1, 1, 0, nullptr, nullptr));
}
SECTION("blockDim.z == maxBlockDimZ") {
const unsigned int z = GetDeviceAttribute(0, hipDeviceAttributeMaxBlockDimZ);
HIP_CHECK(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, z, 1, 1, 0, nullptr, nullptr));
}
}
/**
* Test Description
* ------------------------
* - Negative parameters test for `hipModuleLaunchCooperativeKernel`.
* Test source
* ------------------------
* - unit/module/hipModuleLaunchCooperativeKernel.cc
* Test requirements
* ------------------------
* - HIP_VERSION >= 5.5
*/
TEST_CASE("Unit_hipModuleLaunchCooperativeKernel_Negative_Parameters") {
if (!DeviceAttributesSupport(0, hipDeviceAttributeCooperativeLaunch)) {
HipTest::HIP_SKIP_TEST("CooperativeLaunch not supported");
return;
}
hipFunction_t f = GetKernel(mg.module(), "NOPKernel");
SECTION("f == nullptr") {
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernel(nullptr, 1, 1, 1, 1, 1, 1, 0, nullptr, nullptr),
hipErrorInvalidResourceHandle);
}
SECTION("gridDim.x == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 0, 1, 1, 1, 1, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("gridDim.y == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 0, 1, 1, 1, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("gridDim.z == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 0, 1, 1, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.x == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 0, 1, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.y == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, 0, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.z == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, 1, 0, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.x > maxBlockDimX") {
const unsigned int x = GetDeviceAttribute(0, hipDeviceAttributeMaxBlockDimX) + 1u;
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, x, 1, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.y > maxBlockDimY") {
const unsigned int y = GetDeviceAttribute(0, hipDeviceAttributeMaxBlockDimY) + 1u;
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, y, 1, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.z > maxBlockDimZ") {
const unsigned int z = GetDeviceAttribute(0, hipDeviceAttributeMaxBlockDimZ) + 1u;
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, 1, z, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("blockDim.x * blockDim.y * blockDim.z > maxThreadsPerBlock") {
const unsigned int max = GetDeviceAttribute(0, hipDeviceAttributeMaxThreadsPerBlock);
const unsigned int dim = std::ceil(std::cbrt(max));
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernel(f, 1, 1, 1, dim, dim, dim, 0, nullptr, nullptr),
hipErrorInvalidValue);
}
#if HT_AMD // Disabled due to defect EXSWHTEC-351
SECTION("sharedMemBytes > maxSharedMemoryPerBlock") {
const unsigned int max = GetDeviceAttribute(0, hipDeviceAttributeMaxSharedMemoryPerBlock) + 1u;
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, 1, 1, max, nullptr, nullptr),
hipErrorInvalidValue);
}
SECTION("Invalid stream") {
hipStream_t stream = nullptr;
HIP_CHECK(hipStreamCreate(&stream));
HIP_CHECK(hipStreamDestroy(stream));
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernel(f, 1, 1, 1, 1, 1, 1, 0, stream, nullptr),
hipErrorInvalidValue);
}
#endif
}
+227
Просмотреть файл
@@ -0,0 +1,227 @@
/*
Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
/**
* @addtogroup hipModuleLaunchCooperativeKernelMultiDevice
* hipModuleLaunchCooperativeKernelMultiDevice
* @{
* @ingroup ModuleTest
* `hipModuleLaunchCooperativeKernelMultiDevice(hipFunctionLaunchParams* launchParamsList, unsigned
* int numDevices, unsigned int flags)` -
* Launches kernels on multiple devices where thread blocks can cooperate and synchronize as they
* execute.
*/
#include <hip_test_common.hh>
#include <resource_guards.hh>
#include <utils.hh>
#include "hip_module_launch_kernel_common.hh"
/**
* Test Description
* ------------------------
* - Tests `hipModuleLaunchCooperativeKernel` for a cooperative kernel with no parameters.
* Test source
* ------------------------
* - unit/module/hipModuleLaunchCooperativeKernelMultiDevice.cc
* Test requirements
* ------------------------
* - HIP_VERSION >= 5.5
*/
TEST_CASE("Unit_hipModuleLaunchCooperativeKernelMultiDevice_Positive_Basic") {
if (!DeviceAttributesSupport(0, hipDeviceAttributeCooperativeLaunch)) {
HipTest::HIP_SKIP_TEST("CooperativeLaunch not supported");
return;
}
hipFunction_t f = GetKernel(mg.module(), "CoopKernel");
const auto device_count = HipTest::getDeviceCount();
std::vector<hipFunctionLaunchParams> params_list(device_count);
int device = 0;
for (auto& params : params_list) {
params.function = f;
params.gridDimX = 1;
params.gridDimY = 1;
params.gridDimZ = 1;
params.blockDimX = 1;
params.blockDimY = 1;
params.blockDimZ = 1;
params.kernelParams = nullptr;
params.sharedMemBytes = 0;
HIP_CHECK(hipSetDevice(device++));
HIP_CHECK(hipStreamCreate(&params.hStream));
}
HIP_CHECK(hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count, 0u));
for (const auto params : params_list) {
HIP_CHECK(hipStreamSynchronize(params.hStream));
}
for (const auto params : params_list) {
HIP_CHECK(hipStreamDestroy(params.hStream));
}
}
/**
* Test Description
* ------------------------
* - Negative parameters test for `hipModuleLaunchCooperativeKernelMultiDevice`.
* Test source
* ------------------------
* - unit/module/hipModuleLaunchCooperativeKernelMultiDevice.cc
* Test requirements
* ------------------------
* - HIP_VERSION >= 5.5
*/
TEST_CASE("Unit_hipModuleLaunchCooperativeKernelMultiDevice_Negative_Parameters") {
if (!DeviceAttributesSupport(0, hipDeviceAttributeCooperativeLaunch)) {
HipTest::HIP_SKIP_TEST("CooperativeLaunch not supported");
return;
}
hipFunction_t f = GetKernel(mg.module(), "CoopKernel");
const auto device_count = HipTest::getDeviceCount();
std::vector<hipFunctionLaunchParams> params_list(device_count);
int device = 0;
for (auto& params : params_list) {
params.function = f;
params.gridDimX = 1;
params.gridDimY = 1;
params.gridDimZ = 1;
params.blockDimX = 1;
params.blockDimY = 1;
params.blockDimZ = 1;
params.kernelParams = nullptr;
params.sharedMemBytes = 0;
HIP_CHECK(hipSetDevice(device++));
HIP_CHECK(hipStreamCreate(&params.hStream));
}
SECTION("launchParamsList == nullptr") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernelMultiDevice(nullptr, device_count, 0u),
hipErrorInvalidValue);
}
SECTION("numDevices == 0") {
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), 0, 0u),
hipErrorInvalidValue);
}
SECTION("numDevices > device count") {
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count + 1, 0u),
hipErrorInvalidValue);
}
SECTION("invalid flags") {
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count, 999),
hipErrorInvalidValue);
}
if (device_count > 1) {
SECTION("launchParamsList.func doesn't match across all devices") {
params_list[1].function = GetKernel(mg.module(), "NOPKernel");
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count, 0u),
hipErrorInvalidValue);
}
SECTION("launchParamsList.gridDim doesn't match across all kernels") {
params_list[1].gridDimX = 2;
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count, 0u),
hipErrorInvalidValue);
}
SECTION("launchParamsList.blockDim doesn't match across all kernels") {
params_list[1].blockDimX = 2;
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count, 0u),
hipErrorInvalidValue);
}
SECTION("launchParamsList.sharedMem doesn't match across all kernels") {
params_list[1].sharedMemBytes = 1024;
HIP_CHECK_ERROR(
hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), device_count, 0u),
hipErrorInvalidValue);
}
}
for (const auto params : params_list) {
HIP_CHECK(hipStreamDestroy(params.hStream));
}
}
/**
* Test Description
* ------------------------
* - Tries running `hipModuleLaunchCooperativeKernelMultiDevice` with multiple kernels on the same
* device.
* Test source
* ------------------------
* - unit/module/hipModuleLaunchCooperativeKernelMultiDevice.cc
* Test requirements
* ------------------------
* - HIP_VERSION >= 5.5
*/
TEST_CASE("Unit_hipModuleLaunchCooperativeKernelMultiDevice_Negative_MultiKernelSameDevice") {
if (!DeviceAttributesSupport(0, hipDeviceAttributeCooperativeLaunch)) {
HipTest::HIP_SKIP_TEST("CooperativeLaunch not supported");
return;
}
hipFunction_t f = GetKernel(mg.module(), "CoopKernel");
HIP_CHECK(hipSetDevice(0));
std::vector<hipFunctionLaunchParams> params_list(2);
for (auto& params : params_list) {
params.function = f;
params.gridDimX = 1;
params.gridDimY = 1;
params.gridDimZ = 1;
params.blockDimX = 1;
params.blockDimY = 1;
params.blockDimZ = 1;
params.kernelParams = nullptr;
params.sharedMemBytes = 0;
HIP_CHECK(hipStreamCreate(&params.hStream));
}
HIP_CHECK_ERROR(hipModuleLaunchCooperativeKernelMultiDevice(params_list.data(), 2, 0u),
hipErrorInvalidValue);
for (const auto params : params_list) {
HIP_CHECK(hipStreamDestroy(params.hStream));
}
}
+6
Просмотреть файл
@@ -20,6 +20,7 @@ THE SOFTWARE.
*/
#include <hip/hip_runtime.h>
#include <hip/hip_cooperative_groups.h>
extern "C" {
__global__ void NOPKernel() {}
@@ -34,4 +35,9 @@ __global__ void Delay(uint32_t interval, const uint32_t ticks_per_ms) {
}
}
}
__global__ void CoopKernel() {
cooperative_groups::grid_group grid = cooperative_groups::this_grid();
grid.sync();
}
}